@@ -3,7 +3,9 @@ package workloadcontroller
 import (
 	"fmt"
 	"reflect"
+	"regexp"
 	"sort"
+	"strconv"
 	"strings"
 
 	"github.com/ghodss/yaml"
@@ -76,7 +78,7 @@ func syncOpenShiftAPIServer_v311_00_to_latest(c OpenShiftAPIServerOperator, orig
 
 	imageImportCAModifiedObject, imageImportCAModified, err := manageOpenShiftAPIServerImageImportCA_v311_00_to_latest(c.openshiftConfigClient, c.kubeClient.CoreV1(), c.eventRecorder)
 	if err != nil {
-		errors = append(errors, fmt.Errorf("%q: %v", "client-ca", err))
+		errors = append(errors, fmt.Errorf("%q: %v", "image-import-ca", err))
 	}
 	if imageImportCAModified {
 		reasonsForForcedRollingUpdate = append(reasonsForForcedRollingUpdate, "modified: "+resourceSelectorForCLI(imageImportCAModifiedObject))
@@ -117,25 +119,25 @@ func syncOpenShiftAPIServer_v311_00_to_latest(c OpenShiftAPIServerOperator, orig
 	}
 
 	// manage status
-	var availableConditionReason string
+	var availableConditionReasons []string
 	var availableConditionMessages []string
 
 	switch {
 	case actualDaemonSet == nil:
-		availableConditionReason = "NoDaemon"
+		availableConditionReasons = append(availableConditionReasons, "NoDaemon")
 		availableConditionMessages = append(availableConditionMessages, "daemonset/apiserver.openshift-apiserver: could not be retrieved")
 	case actualDaemonSet.Status.NumberAvailable == 0:
-		availableConditionReason = "NoAPIServerPod"
+		availableConditionReasons = append(availableConditionReasons, "NoAPIServerPod")
 		availableConditionMessages = append(availableConditionMessages, "no openshift-apiserver daemon pods available on any node.")
 	case actualDaemonSet.Status.NumberAvailable > 0 && len(actualAPIServices) == 0:
-		availableConditionReason = "NoRegisteredAPIServices"
+		availableConditionReasons = append(availableConditionReasons, "NoRegisteredAPIServices")
 		availableConditionMessages = append(availableConditionMessages, "registered apiservices could not be retrieved")
 	}
 	for _, apiService := range actualAPIServices {
 		for _, condition := range apiService.Status.Conditions {
 			if condition.Type == apiregistrationv1.Available {
 				if condition.Status == apiregistrationv1.ConditionFalse {
-					availableConditionReason = "APIServiceNotAvailable"
+					availableConditionReasons = append(availableConditionReasons, "APIServiceNotAvailable")
 					availableConditionMessages = append(availableConditionMessages, fmt.Sprintf("apiservice/%v: not available: %v", apiService.Name, condition.Message))
 				}
 				break
@@ -146,22 +148,25 @@ func syncOpenShiftAPIServer_v311_00_to_latest(c OpenShiftAPIServerOperator, orig
 	// of something delaying them.  This isn't perfect because of round-robining, but let's see if we get an improvement
 	if len(availableConditionMessages) == 0 && c.kubeClient.Discovery().RESTClient() != nil {
 		missingAPIMessages := checkForAPIs(c.eventRecorder, c.kubeClient.Discovery().RESTClient(), apiServiceGroupVersions...)
+		availableConditionReasons = append(availableConditionReasons, "OpenShiftAPICheckFailed")
 		availableConditionMessages = append(availableConditionMessages, missingAPIMessages...)
 	}
 
+	sort.Sort(sort.StringSlice(availableConditionReasons))
+
 	switch {
 	case len(availableConditionMessages) == 1:
 		v1helpers.SetOperatorCondition(&operatorConfig.Status.Conditions, operatorv1.OperatorCondition{
 			Type:    operatorv1.OperatorStatusTypeAvailable,
 			Status:  operatorv1.ConditionFalse,
-			Reason:  availableConditionReason,
+			Reason:  availableConditionReasons[0],
 			Message: availableConditionMessages[0],
 		})
 	case len(availableConditionMessages) > 1:
 		v1helpers.SetOperatorCondition(&operatorConfig.Status.Conditions, operatorv1.OperatorCondition{
 			Type:    operatorv1.OperatorStatusTypeAvailable,
 			Status:  operatorv1.ConditionFalse,
-			Reason:  "Multiple",
+			Reason:  strings.Join(availableConditionReasons, "\n"),
 			Message: strings.Join(availableConditionMessages, "\n"),
 		})
 	default:
@@ -365,35 +370,22 @@ func manageOpenShiftAPIServerDaemonSet_v311_00_to_latest(
 	operatorConfig *operatorv1.OpenShiftAPIServer,
 	generationStatus []operatorv1.GenerationStatus,
 	forceRollingUpdate bool) (*appsv1.DaemonSet, bool, error) {
-	required := resourceread.ReadDaemonSetV1OrDie(v311_00_assets.MustAsset("v3.11.0/openshift-apiserver/ds.yaml"))
+	tmpl := v311_00_assets.MustAsset("v3.11.0/openshift-apiserver/ds.yaml")
 
-	for i := range required.Spec.Template.Spec.Containers {
-		img := strings.TrimSpace(required.Spec.Template.Spec.Containers[i].Image)
-		switch img {
-		case "${IMAGE}":
-			required.Spec.Template.Spec.Containers[i].Image = imagePullSpec
-		case "${OPERATOR_IMAGE}":
-			required.Spec.Template.Spec.Containers[i].Image = operatorImagePullSpec
-		default:
-			if strings.Contains(img, "$") {
-				return nil, false, fmt.Errorf("invalid image reference %q", img)
-			}
-		}
-	}
-	for i := range required.Spec.Template.Spec.InitContainers {
-		img := strings.TrimSpace(required.Spec.Template.Spec.InitContainers[i].Image)
-		switch img {
-		case "${IMAGE}":
-			required.Spec.Template.Spec.InitContainers[i].Image = imagePullSpec
-		case "${OPERATOR_IMAGE}":
-			required.Spec.Template.Spec.InitContainers[i].Image = operatorImagePullSpec
-		default:
-			if strings.Contains(img, "$") {
-				return nil, false, fmt.Errorf("invalid image reference %q", img)
-			}
-		}
+	r := strings.NewReplacer(
+		"${IMAGE}", imagePullSpec,
+		"${OPERATOR_IMAGE}", operatorImagePullSpec,
+		"${REVISION}", strconv.Itoa(int(operatorConfig.Status.LatestAvailableRevision)),
+	)
+	tmpl = []byte(r.Replace(string(tmpl)))
+
+	re := regexp.MustCompile("\\$\\{[^}]*}")
+	if match := re.Find(tmpl); len(match) > 0 {
+		return nil, false, fmt.Errorf("invalid template reference %q", string(match))
 	}
 
+	required := resourceread.ReadDaemonSetV1OrDie(tmpl)
+
 	// we set this so that when the requested image pull spec changes, we always have a diff.  Remember that we don't directly
 	// diff any fields on the daemonset because they can be rewritten by admission and we don't want to constantly be fighting
 	// against admission or defaults.  That was a problem with original versions of apply.
@@ -403,6 +395,9 @@ func manageOpenShiftAPIServerDaemonSet_v311_00_to_latest(
 	required.Annotations["openshiftapiservers.operator.openshift.io/pull-spec"] = imagePullSpec
 	required.Annotations["openshiftapiservers.operator.openshift.io/operator-pull-spec"] = operatorImagePullSpec
 
+	required.Labels["revision"] = strconv.Itoa(int(operatorConfig.Status.LatestAvailableRevision))
+	required.Spec.Template.Labels["revision"] = strconv.Itoa(int(operatorConfig.Status.LatestAvailableRevision))
+
 	containerArgsWithLoglevel := required.Spec.Template.Spec.Containers[0].Args
 	if argsCount := len(containerArgsWithLoglevel); argsCount > 1 {
 		return nil, false, fmt.Errorf("expected only one container argument, got %d", argsCount)
@@ -463,6 +458,7 @@ func manageAPIServices_v311_00_to_latest(client apiregistrationv1client.APIServi
 				VersionPriority:      15,
 			},
 		}
+		apiregistrationv1.SetDefaults_ServiceReference(obj.Spec.Service)
 
 		apiService, _, err := resourceapply.ApplyAPIService(client, recorder, obj)
 		if err != nil {
