@@ -22,6 +22,7 @@ import (
 	kubetesting "k8s.io/client-go/testing"
 	apiregistrationv1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1"
 	kubeaggregatorfake "k8s.io/kube-aggregator/pkg/client/clientset_generated/clientset/fake"
+	utilpointer "k8s.io/utils/pointer"
 
 	operatorv1 "github.com/openshift/api/operator/v1"
 	configfake "github.com/openshift/client-go/config/clientset/versioned/fake"
@@ -173,11 +174,11 @@ func TestProgressingCondition(t *testing.T) {
 }
 
 func TestAvailableStatus(t *testing.T) {
-
+	servicePort := utilpointer.Int32Ptr(443)
 	testCases := []struct {
 		name                    string
 		expectedStatus          operatorv1.ConditionStatus
-		expectedReason          string
+		expectedReasons         []string
 		expectedMessages        []string
 		expectedFailingMessages []string
 		apiServiceReactor       kubetesting.ReactionFunc
@@ -190,7 +191,7 @@ func TestAvailableStatus(t *testing.T) {
 		{
 			name:                    "APIServiceCreateFailure",
 			expectedStatus:          operatorv1.ConditionFalse,
-			expectedReason:          "NoRegisteredAPIServices",
+			expectedReasons:         []string{"NoRegisteredAPIServices"},
 			expectedMessages:        []string{"registered apiservices could not be retrieved"},
 			expectedFailingMessages: []string{"\"apiservices\": TEST ERROR: fail to create apiservice"},
 
@@ -210,7 +211,7 @@ func TestAvailableStatus(t *testing.T) {
 		{
 			name:                    "APIServiceGetFailure",
 			expectedStatus:          operatorv1.ConditionFalse,
-			expectedReason:          "NoRegisteredAPIServices",
+			expectedReasons:         []string{"NoRegisteredAPIServices"},
 			expectedMessages:        []string{"registered apiservices could not be retrieved"},
 			expectedFailingMessages: []string{"\"apiservices\": TEST ERROR: fail to get apiservice"},
 
@@ -224,7 +225,7 @@ func TestAvailableStatus(t *testing.T) {
 		{
 			name:                    "DaemonSetGetFailure",
 			expectedStatus:          operatorv1.ConditionFalse,
-			expectedReason:          "NoDaemon",
+			expectedReasons:         []string{"NoDaemon"},
 			expectedMessages:        []string{"daemonset/apiserver.openshift-apiserver: could not be retrieved"},
 			expectedFailingMessages: []string{"\"daemonsets\": TEST ERROR: fail to get daemonset/apiserver.openshift-apiserver"},
 
@@ -238,7 +239,7 @@ func TestAvailableStatus(t *testing.T) {
 		{
 			name:             "NoDaemonSetPods",
 			expectedStatus:   operatorv1.ConditionFalse,
-			expectedReason:   "NoAPIServerPod",
+			expectedReasons:  []string{"NoAPIServerPod"},
 			expectedMessages: []string{"no openshift-apiserver daemon pods available on any node."},
 
 			daemonReactor: func(action kubetesting.Action) (handled bool, ret runtime.Object, err error) {
@@ -254,7 +255,7 @@ func TestAvailableStatus(t *testing.T) {
 		{
 			name:             "APIServiceNotAvailable",
 			expectedStatus:   operatorv1.ConditionFalse,
-			expectedReason:   "APIServiceNotAvailable",
+			expectedReasons:  []string{"APIServiceNotAvailable"},
 			expectedMessages: []string{"apiservice/v1.build.openshift.io: not available: TEST MESSAGE"},
 
 			apiServiceReactor: func(action kubetesting.Action) (handled bool, ret runtime.Object, err error) {
@@ -264,7 +265,7 @@ func TestAvailableStatus(t *testing.T) {
 						Spec: apiregistrationv1.APIServiceSpec{
 							Group:                "build.openshift.io",
 							Version:              "v1",
-							Service:              &apiregistrationv1.ServiceReference{Namespace: operatorclient.TargetNamespace, Name: "api"},
+							Service:              &apiregistrationv1.ServiceReference{Namespace: operatorclient.TargetNamespace, Name: "api", Port: servicePort},
 							GroupPriorityMinimum: 9900,
 							VersionPriority:      15,
 						},
@@ -279,9 +280,9 @@ func TestAvailableStatus(t *testing.T) {
 			},
 		},
 		{
-			name:           "MultipleAPIServiceNotAvailable",
-			expectedStatus: operatorv1.ConditionFalse,
-			expectedReason: "Multiple",
+			name:            "MultipleAPIServiceNotAvailable",
+			expectedStatus:  operatorv1.ConditionFalse,
+			expectedReasons: []string{"APIServiceNotAvailable", "APIServiceNotAvailable"},
 			expectedMessages: []string{
 				"apiservice/v1.build.openshift.io: not available: TEST MESSAGE",
 				"apiservice/v1.project.openshift.io: not available: TEST MESSAGE",
@@ -301,7 +302,7 @@ func TestAvailableStatus(t *testing.T) {
 						Spec: apiregistrationv1.APIServiceSpec{
 							Group:                action.GetResource().Group,
 							Version:              action.GetResource().Version,
-							Service:              &apiregistrationv1.ServiceReference{Namespace: operatorclient.TargetNamespace, Name: "api"},
+							Service:              &apiregistrationv1.ServiceReference{Namespace: operatorclient.TargetNamespace, Name: "api", Port: servicePort},
 							GroupPriorityMinimum: 9900,
 							VersionPriority:      15,
 						},
@@ -357,7 +358,7 @@ func TestAvailableStatus(t *testing.T) {
 				return true,
 					&apiregistrationv1.APIService{
 						ObjectMeta: metav1.ObjectMeta{Name: action.(kubetesting.GetAction).GetName(), Annotations: map[string]string{"service.alpha.openshift.io/inject-cabundle": "true"}},
-						Spec:       apiregistrationv1.APIServiceSpec{Group: action.GetResource().Group, Version: action.GetResource().Version, Service: &apiregistrationv1.ServiceReference{Namespace: operatorclient.TargetNamespace, Name: "api"}, GroupPriorityMinimum: 9900, VersionPriority: 15},
+						Spec:       apiregistrationv1.APIServiceSpec{Group: action.GetResource().Group, Version: action.GetResource().Version, Service: &apiregistrationv1.ServiceReference{Namespace: operatorclient.TargetNamespace, Name: "api", Port: servicePort}, GroupPriorityMinimum: 9900, VersionPriority: 15},
 						Status:     apiregistrationv1.APIServiceStatus{Conditions: []apiregistrationv1.APIServiceCondition{{Type: apiregistrationv1.Available, Status: apiregistrationv1.ConditionTrue}}},
 					}, nil
 			})
@@ -392,8 +393,9 @@ func TestAvailableStatus(t *testing.T) {
 			if condition.Status != tc.expectedStatus {
 				t.Error(diff.ObjectGoPrintSideBySide(condition.Status, tc.expectedStatus))
 			}
-			if tc.expectedReason != "" && condition.Reason != tc.expectedReason {
-				t.Error(diff.ObjectGoPrintSideBySide(condition.Reason, tc.expectedReason))
+			expectedReasons := strings.Join(tc.expectedReasons, "\n")
+			if len(expectedReasons) > 0 && condition.Reason != expectedReasons {
+				t.Error(diff.ObjectGoPrintSideBySide(condition.Reason, expectedReasons))
 			}
 			if len(tc.expectedMessages) > 0 {
 				actualMessages := strings.Split(condition.Message, "\n")
